Attribute VB_Name = "Livelli"
'======================================================
' Descrizione.....: Subroutine CONREC
' Nome dei Files..: Livelli.bas
' Data............: 26/7/1999
' Versione........: 1.0 a 32 bits.
' Sistema.........: Visual Basic 6.0 sotto Windows NT.
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
'======================================================
'
'   Subroutine CONREC per il tracciamento delle curve di livello:
'
'   I valori della superficie da tracciare devono essere passati
'   nella matrice D(0 to IUB, 0 to JUB): questa rappresenta una
'   griglia con colonne da 0 a IUB e righe da 0 a JUB.
'
'   Le coordinate reali di colonne e righe, alle quali corrispondono
'   i valori di livello della matrice D(.., ..), vengono passate nei
'   vettori X(0 to IUB), Y(0 to JUB): queste non devono necessariamente
'   essere equispaziate e normalmente, corrispondono alla scalatura
'   del foglio (Page) su cui si vuole disegnare.
'
'   Gli NL valori a cui si intendono tracciare le linee di livello
'   vengono passati nel vettore Z(0 to NL - 1): questi valori
'   non devono necessariamente essere equispaziati ma devono essere
'   in ordine crescente.
'   Nel programma di prova (frmCaleido) e' stato scelto MaxNL = 15
'   per avere ogni linea tracciata con un colore diverso di QBColor
'   (16 colori meno il nero). Volendo si potrebbe passare a CONREC
'   un ulteriore vettore (e.g. ZLTipo(0 to NL - 1)) che fornisca,
'   per ogni livello, colore e/o tipo di linea. Modificare in questo
'   caso l' istruzione DRAWIT: della Sub CONREC.
'
'   Page e' il Picture Box sul quale si vuole disegnare: puo' essere,
'   anziche' un PictureBox, qualsiasi oggetto che supporti il metodo
'   Line.
'
'   Msg$ viene ritornata vuota se CONREC non ha trovato errori,
'   altrimenti contiene la descrizione dell' errore.
'
'   L' istruzione CANC: di CONREC non e' normalmente necessaria:
'   serve solo in questa versione dimostrativa per le chiamate
'   successive, ad effetto grafico, di frmCaleido.
'
'   Per l' uso, la scalatura del foglio e la preparazione del
'   vettore dei livelli vedere la routine cmbFunzione_Click e le
'   routines Cos0, Cos1, etc.. di frmCaleido.
'
Option Explicit

Sub CONREC(ByVal Page As PictureBox, D!(), X!(), Y!(), Z!(), Msg$)
'
'   Subroutine CONREC
'   Da: A countouring subroutine.
'   Di: P. Bourke
'   Byte 1/Jun/1987
'
'   Input Variables to CONREC:
'
'   IUB&, JUB&               Index bounds of the Data Array.
'   D!(0 to IUB&,0 to JUB&)  Matrix for the Data Surface.
'   X!(0 to IUB&)            Data Array for Column coordinates.
'   Y!(0 to JUB&)            Data Array for Row    coordinates.
'   NL&                      Number of Contour Levels.
'   Z!(0 to NL& - 1)         Contour Levels in increasing order.
'
    Dim IUB&, JUB&, NL_1&
    Dim K&, J&, I&, M&, M1&, M2&, M3&, CAS&
    Dim X1!, Y1!, X2!, Y2!, DMin!, DMax!
'
    Dim H!(0 To 4), XH!(0 To 4), YH!(0 To 4), ISH&(0 To 4)
'
    Dim IM&(0 To 3)
    IM(0) = 0: IM(1) = 1: IM(2) = 1:  IM(3) = 0
'
    Dim JM&(0 To 3)
    JM(0) = 0: JM(1) = 0: JM(2) = 1: JM(3) = 1
'
    Dim CT&(0 To 2, 0 To 2, 0 To 2)
    CT(0, 0, 0) = 0: CT(0, 0, 1) = 0: CT(0, 0, 2) = 8
    CT(0, 1, 0) = 0: CT(0, 1, 1) = 2: CT(0, 1, 2) = 5
    CT(0, 2, 0) = 7: CT(0, 2, 1) = 6: CT(0, 2, 2) = 9
    CT(1, 0, 0) = 0: CT(1, 0, 1) = 3: CT(1, 0, 2) = 4
    CT(1, 1, 0) = 1: CT(1, 1, 1) = 3: CT(1, 1, 2) = 1
    CT(1, 2, 0) = 4: CT(1, 2, 1) = 3: CT(1, 2, 2) = 0
    CT(2, 0, 0) = 9: CT(2, 0, 1) = 6: CT(2, 0, 2) = 7
    CT(2, 1, 0) = 5: CT(2, 1, 1) = 2: CT(2, 1, 2) = 0
    CT(2, 2, 0) = 8: CT(2, 2, 1) = 0: CT(2, 2, 2) = 0
'
    IUB = UBound(X)
    JUB = UBound(Y)
    NL_1 = UBound(Z)    ' NL - 1
'
    On Error GoTo CONREC_ERR
    If IUB <= 0 Or JUB <= 0 Or NL_1 <= 0 Then Err.Raise 1001, "CONREC", "Error in Input Parameters"
    For K = 1 To NL_1
        If Z(K) <= Z(K - 1) Then
            Err.Raise 1001, "CONREC", "Error in Input Parameters"
            Exit For
        End If
    Next K
'
'
    For J = 0 To JUB - 1
        For I = 0 To IUB - 1
'
            ' Cancella solo la porzione di Area interessata dal Loop
            ' in corso. E' preferibile invece cancellare inizialmente
            ' tutta la Pagina Grafica per evitare Linee di Livello
            ' spezzate.
CANC:       Page.Line (X(I), Y(J))-(X(I + 1), Y(J + 1)), Page.BackColor, BF
'
            If D(I, J) < D(I, J + 1) Then
                DMin = D(I, J)
            Else
                DMin = D(I, J + 1)
            End If
            If D(I + 1, J) < DMin Then DMin = D(I + 1, J)
            If D(I + 1, J + 1) < DMin Then DMin = D(I + 1, J + 1)
'
            If D(I, J) > D(I, J + 1) Then
                DMax = D(I, J)
            Else
                DMax = D(I, J + 1)
            End If
            If D(I + 1, J) > DMax Then DMax = D(I + 1, J)
            If D(I + 1, J + 1) > DMax Then DMax = D(I + 1, J + 1)
'
            If DMax < Z(0) Or DMin > Z(NL_1) Or DMax = DMin Then GoTo NOIBOX
'
            For K = 0 To NL_1
                If Z(K) < DMin Or Z(K) > DMax Then GoTo NOITRI
'
                For M = 4 To 0 Step -1
                    If M > 0 Then
                        H(M) = D(I + IM(M - 1), J + JM(M - 1)) - Z(K)
                        XH(M) = X(I + IM(M - 1))
                        YH(M) = Y(J + JM(M - 1))
                    End If
                    If M = 0 Then
                        H(0) = (H(1) + H(2) + H(3) + H(4)) / 4!
                        XH(0) = (X(I) + X(I + 1)) / 2!
                        YH(0) = (Y(J) + Y(J + 1)) / 2!
                    End If
                    If H(M) > 0 Then ISH(M) = 2
                    If H(M) < 0 Then ISH(M) = 0
                    If H(M) = 0 Then ISH(M) = 1
                Next M
'
                For M = 1 To 4
                    M1 = M
                    M2 = 0
                    M3 = M + 1
                    If M3 = 5 Then M3 = 1
                    CAS = CT(ISH(M1), ISH(M2), ISH(M3))
                    If CAS = 0 Then GoTo CASE0
                    Select Case CAS
                        Case 1
                        X1 = XH(M1)
                        Y1 = YH(M1)
                        X2 = XH(M2)
                        Y2 = YH(M2)
'
                        Case 2
                        X1 = XH(M2)
                        Y1 = YH(M2)
                        X2 = XH(M3)
                        Y2 = YH(M3)
'
                        Case 3
                        X1 = XH(M3)
                        Y1 = YH(M3)
                        X2 = XH(M1)
                        Y2 = YH(M1)
'
                        Case 4
                        X1 = XH(M1)
                        Y1 = YH(M1)
                        X2 = (H(M3) * XH(M2) - H(M2) * XH(M3)) / (H(M3) - H(M2))
                        Y2 = (H(M3) * YH(M2) - H(M2) * YH(M3)) / (H(M3) - H(M2))
'
                        Case 5
                        X1 = XH(M2)
                        Y1 = YH(M2)
                        X2 = (H(M1) * XH(M3) - H(M3) * XH(M1)) / (H(M1) - H(M3))
                        Y2 = (H(M1) * YH(M3) - H(M3) * YH(M1)) / (H(M1) - H(M3))
'
                        Case 6
                        X1 = XH(M3)
                        Y1 = YH(M3)
                        X2 = (H(M2) * XH(M1) - H(M1) * XH(M2)) / (H(M2) - H(M1))
                        Y2 = (H(M2) * YH(M1) - H(M1) * YH(M2)) / (H(M2) - H(M1))
'
                        Case 7
                        X1 = (H(M2) * XH(M1) - H(M1) * XH(M2)) / (H(M2) - H(M1))
                        Y1 = (H(M2) * YH(M1) - H(M1) * YH(M2)) / (H(M2) - H(M1))
                        X2 = (H(M3) * XH(M2) - H(M2) * XH(M3)) / (H(M3) - H(M2))
                        Y2 = (H(M3) * YH(M2) - H(M2) * YH(M3)) / (H(M3) - H(M2))
'
                        Case 8
                        X1 = (H(M3) * XH(M2) - H(M2) * XH(M3)) / (H(M3) - H(M2))
                        Y1 = (H(M3) * YH(M2) - H(M2) * YH(M3)) / (H(M3) - H(M2))
                        X2 = (H(M1) * XH(M3) - H(M3) * XH(M1)) / (H(M1) - H(M3))
                        Y2 = (H(M1) * YH(M3) - H(M3) * YH(M1)) / (H(M1) - H(M3))
'
                        Case 9
                        X1 = (H(M1) * XH(M3) - H(M3) * XH(M1)) / (H(M1) - H(M3))
                        Y1 = (H(M1) * YH(M3) - H(M3) * YH(M1)) / (H(M1) - H(M3))
                        X2 = (H(M2) * XH(M1) - H(M1) * XH(M2)) / (H(M2) - H(M1))
                        Y2 = (H(M2) * YH(M1) - H(M1) * YH(M2)) / (H(M2) - H(M1))
                    End Select
'
DRAWIT:             Page.Line (X1, Y1)-(X2, Y2), QBColor(K + 1)
                    DoEvents
                    ' Gestione di un eventuale Flag di fine ciclo:
                    If frmCaleido.FStop Then Exit Sub
'
CASE0:          Next M
'
NOITRI:     Next K
'
NOIBOX: Next I
'
    Next J
'
CONREC_ERR:
    Msg$ = IIf(Err = 0, "", Err.Description)
'
'
'
End Sub

